###############################################################################
 #  Copyright (c) 2018, Xilinx, Inc.
 #  All rights reserved.
 #
 #  Redistribution and use in source and binary forms, with or without
 #  modification, are permitted provided that the following conditions are met:
 #
 #  1.  Redistributions of source code must retain the above copyright notice,
 #     this list of conditions and the following disclaimer.
 #
 #  2.  Redistributions in binary form must reproduce the above copyright
 #      notice, this list of conditions and the following disclaimer in the
 #      documentation and/or other materials provided with the distribution.
 #
 #  3.  Neither the name of the copyright holder nor the names of its
 #      contributors may be used to endorse or promote products derived from
 #      this software without specific prior written permission.
 #
 #  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 #  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 #  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 #  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 #  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 #  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 #  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 #  OR BUSINESS INTERRUPTION). HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 #  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 #  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 #  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
###############################################################################
 # @file create_proj.tcl
 #
 #	Tcl script for re-creating project AXI Basic Series 4
 # 
 #
###############################################################################

################################################################
# Check if script is running in correct Vivado version.
################################################################
set scripts_vivado_version "2019.2"
set current_vivado_version [version -short]

if { [string first $current_vivado_version $scripts_vivado_version] == -1 } {
   puts "The version $current_vivado_version is not supported. Supported versions are $scripts_vivado_version"
   return 1
}

# Configuration - Can be modified by the user
set project_name 				AXI_Basics_4

# Create a new project
create_project $project_name ./$project_name -part xc7z020clg484-1
set_property target_language Verilog [current_project]

# Create 'sources_1' fileset (if not found)
if {[string equal [get_filesets -quiet sources_1] ""]} {
  create_fileset -srcset sources_1
}
# Add hdl files
add_files -fileset sources_1 -norecurse -scan_for_includes ./src/hdl
import_files -fileset sources_1 -norecurse ./src/hdl

# Create 'constrs_1' fileset (if not found)
#if {[string equal [get_filesets -quiet constrs_1] ""]} {
#  create_fileset -constrset -quiet constrs_1
#}
# Add constraint files
#add_files -fileset constrs_1 -norecurse -scan_for_includes ./src/constr
#import_files -fileset constrs_1 -norecurse ./src/constr

# Create 'sim_1' fileset (if not found)
if {[string equal [get_filesets -quiet sim_1] ""]} {
  create_fileset -simset sim_1
}
# Add simulation files
add_files -fileset sim_1 -norecurse -scan_for_includes ./src/sim
import_files -fileset sim_1 -norecurse ./src/sim


# Add ip repository
#set_property  ip_repo_paths  ./src/ip_repo [current_project]
#update_ip_catalog

set BD_name 	design_1

#Create the Block Design
create_bd_design $BD_name
common::send_msg_id "BD_TCL-004" "INFO" "Making design <$BD_name> as current_bd_design."
current_bd_design $BD_name

set axi_master_0 [create_bd_cell -type module -reference AXI_Master_v1_0 axi_master_0]

# Create ports
set aclk [ create_bd_port -dir I -type clk aclk -freq_hz 100000000]
set aresetn [ create_bd_port -dir I -type rst aresetn ]
set txn [ create_bd_port -dir I txn ]


# Create port connections
connect_bd_net -net aclk_0_0 	[get_bd_ports aclk] [get_bd_pins axi_master_0/m00_axi_aclk]
connect_bd_net -net aresetn_0_0 [get_bd_ports aresetn] [get_bd_pins axi_master_0/m00_axi_aresetn]
connect_bd_net -net txn_0_0 	[get_bd_ports txn] [get_bd_pins axi_master_0/m00_axi_init_axi_txn]

set_property -dict [list CONFIG.C_M00_AXI_TARGET_SLAVE_BASE_ADDR {0x44A00FC8}] [get_bd_cells axi_master_0]

# Validate the BD
regenerate_bd_layout
validate_bd_design 
save_bd_design

#Generate the wrapper
make_wrapper -files [get_files ${BD_name}.bd] -top

# Add the wrapper to the fileset
set obj [get_filesets sources_1]
set files [list "[file normalize [glob "./$project_name/$project_name.srcs/sources_1/bd/$BD_name/hdl/${BD_name}_wrapper.v"]]"]
add_files -norecurse -fileset $obj $files

set_property top ${BD_name}_wrapper [current_fileset]
update_compile_order -fileset sources_1

set_property top AXI_tb [get_filesets sim_1]
set_property top_lib xil_defaultlib [get_filesets sim_1]
update_compile_order -fileset sim_1


# Generate the output products
#generate_target all [get_files ./$project_name/$project_name.srcs/sources_1/bd/$BD_name/${BD_name}.bd]
#create_ip_run [get_files -of_objects [get_fileset sources_1] ./$project_name/$project_name.srcs/sources_1/bd/$BD_name/${BD_name}.bd]
#launch_runs -jobs 8 [get_runs $BD_name*synth_1]